@@ -468,7 +468,7 @@ impl<'a> From<IriRef<&'a str>> for IriRef<Cow<'a, str>> {
     }
 }
 
-impl<'a> From<IriRef<String>> for IriRef<Cow<'a, str>> {
+impl From<IriRef<String>> for IriRef<Cow<'_, str>> {
     #[inline]
     fn from(iri: IriRef<String>) -> Self {
         Self {
@@ -510,7 +510,7 @@ impl<'de, T: Deref<Target = str> + Deserialize<'de>> Deserialize<'de> for IriRef
     fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
         use serde::de::Error;
 
-        Self::parse(T::deserialize(deserializer)?).map_err(D::Error::custom)
+        Self::parse(T::deserialize(deserializer)?).map_err(Error::custom)
     }
 }
 
@@ -662,7 +662,7 @@ impl<T: Deref<Target = str>> Iri<T> {
         let base_query = base.query();
 
         // We validate the path, resolving algorithm eats /. and /.. in hierarchical path
-        for segment in abs_path.split('/').skip(1) {
+        for segment in abs_path.split('/') {
             if matches!(segment, "." | "..") {
                 return Err(IriRelativizeError {});
             }
@@ -703,17 +703,17 @@ impl<T: Deref<Target = str>> Iri<T> {
         }
         if abs_path != base_path || abs_query.is_none() && base_query.is_some() {
             let number_of_shared_characters = abs_path
-                .bytes()
-                .zip(base_path.bytes())
+                .chars()
+                .zip(base_path.chars())
                 .take_while(|(l, r)| l == r)
-                .count();
+                .map(|(l, _)| l.len_utf8())
+                .sum::<usize>();
             // We decrease until finding a /
             let number_of_shared_characters = abs_path[..number_of_shared_characters]
                 .rfind('/')
                 .map_or(0, |n| n + 1);
-            return if abs_path[number_of_shared_characters..].contains('/')
+            return if abs_path[number_of_shared_characters..].starts_with('/')
                 || base_path[number_of_shared_characters..].contains('/')
-                || abs_path[number_of_shared_characters..].is_empty()
                 || abs_path[number_of_shared_characters..].contains(':')
             {
                 // We output the full path because we have a / or an empty end
@@ -735,6 +735,28 @@ impl<T: Deref<Target = str>> Iri<T> {
                         },
                     })
                 }
+            } else if abs_path[number_of_shared_characters..].is_empty() {
+                // We use "."
+                let mut iri = String::with_capacity(
+                    abs.0.len() - abs.0.positions.authority_end - number_of_shared_characters + 1,
+                );
+                iri.push('.');
+                iri.push_str(&abs.0[abs.0.positions.authority_end + number_of_shared_characters..]);
+                Ok(IriRef {
+                    iri,
+                    positions: IriElementsPositions {
+                        scheme_end: 0,
+                        authority_end: 0,
+                        path_end: abs.0.positions.path_end
+                            - abs.0.positions.authority_end
+                            - number_of_shared_characters
+                            + 1,
+                        query_end: abs.0.positions.query_end
+                            - abs.0.positions.authority_end
+                            - number_of_shared_characters
+                            + 1,
+                    },
+                })
             } else {
                 // We just override the last element
                 Ok(IriRef {
@@ -1078,7 +1100,7 @@ impl<'a> From<Iri<&'a str>> for Iri<Cow<'a, str>> {
     }
 }
 
-impl<'a> From<Iri<String>> for Iri<Cow<'a, str>> {
+impl From<Iri<String>> for Iri<Cow<'_, str>> {
     #[inline]
     fn from(iri: Iri<String>) -> Self {
         Self(iri.0.into())
@@ -1132,7 +1154,7 @@ impl<'de, T: Deref<Target = str> + Deserialize<'de>> Deserialize<'de> for Iri<T>
         use serde::de::Error;
         IriRef::deserialize(deserializer)?
             .try_into()
-            .map_err(D::Error::custom)
+            .map_err(Error::custom)
     }
 }
 
@@ -1159,6 +1181,9 @@ impl fmt::Display for IriParseError {
                 "Invalid IRI percent encoding '{}'",
                 cs.iter().flatten().cloned().collect::<String>()
             ),
+            IriParseErrorKind::PathStartingWithTwoSlashes => {
+                write!(f, "An IRI path is not allowed to start with //")
+            }
         }
     }
 }
@@ -1181,6 +1206,7 @@ enum IriParseErrorKind {
     InvalidPortCharacter(char),
     InvalidIriCodePoint(char),
     InvalidPercentEncoding([Option<char>; 3]),
+    PathStartingWithTwoSlashes,
 }
 
 /// An error raised when calling [`Iri::relativize`].
@@ -1297,7 +1323,7 @@ struct ParserInput<'a> {
     position: usize,
 }
 
-impl<'a> ParserInput<'a> {
+impl ParserInput<'_> {
     #[inline]
     fn next(&mut self) -> Option<char> {
         if let Some(head) = self.value.next() {
@@ -1644,13 +1670,34 @@ impl<'a, O: OutputBuffer, const UNCHECKED: bool> IriParser<'a, O, UNCHECKED> {
             let c = self.input.next();
             match c {
                 None | Some('/') | Some('?') | Some('#') => {
-                    if self.output.as_str().ends_with("/..") {
-                        self.output.truncate(self.output.len() - 3);
-                        self.remove_last_segment();
-                    } else if self.output.as_str().ends_with("/.") {
-                        self.output.truncate(self.output.len() - 1);
+                    let output_str = self.output.as_str();
+                    if !output_str.is_empty() {
+                        let output_path = &output_str[self.output_positions.authority_end..];
+                        if output_path.ends_with("/..") {
+                            self.output.truncate(self.output.len() - 3);
+                            self.remove_last_segment();
+                        } else if output_path.ends_with("/.") || output_path == "." {
+                            self.output.truncate(self.output.len() - 1);
+                        } else if output_path == ".." {
+                            self.output.truncate(self.output.len() - 2);
+                        } else if c == Some('/') {
+                            self.output.push('/');
+                            continue;
+                        }
                     } else if c == Some('/') {
                         self.output.push('/');
+                        continue;
+                    }
+
+                    // We validate that the path does not start with "//" and is ambiguous with an authority
+                    // It can happen after resolving ./ and ../
+                    let output_str = self.output.as_str();
+                    if !UNCHECKED
+                        && !output_str.is_empty()
+                        && output_str[self.output_positions.authority_end..].starts_with("//")
+                        && self.output_positions.authority_end == self.output_positions.scheme_end
+                    {
+                        return self.parse_error(IriParseErrorKind::PathStartingWithTwoSlashes);
                     }
 
                     if c == Some('?') {
