@@ -276,6 +276,17 @@ fn test_wrong_relative_parsing() {
     }
 }
 
+#[test]
+fn test_wrong_relative_parsing_on_scheme() {
+    let examples = [".///C:::"];
+
+    let base = Iri::parse("x:").unwrap();
+    for e in examples {
+        let result = base.resolve(e);
+        assert!(result.is_err(), "{} is wrongly considered valid", e);
+    }
+}
+
 #[test]
 fn test_resolve_relative_iri() {
     // From https://sourceforge.net/projects/foursuite/ under Apache License
@@ -495,11 +506,11 @@ fn test_resolve_relative_iri() {
         ("", "foo:a/b?c#d", "foo:a/b?c"),
         ("b/c", "foo:a", "foo:b/c"),
         ("../b/c", "foo:/a/y/z", "foo:/a/b/c"),
-        //("./b/c", "foo:a", "foo:b/c"),
-        //("/./b/c", "foo:a", "foo:/b/c"),
+        ("./b/c", "foo:a", "foo:b/c"),
+        ("/./b/c", "foo:a", "foo:/b/c"),
         ("../../d", "foo://a//b/c", "foo://a/d"),
-        //(".", "foo:a", "foo:"),
-        //("..", "foo:a", "foo:"),
+        (".", "foo:a", "foo:"),
+        ("..", "foo:a", "foo:"),
         // 50-57 (cf. TimBL comments --
         //  http://lists.w3.org/Archives/Public/uri/2003Feb/0028.html,
         //  http://lists.w3.org/Archives/Public/uri/2003Jan/0008.html)
@@ -554,16 +565,30 @@ fn test_resolve_relative_iri() {
         ("///lv2.h", "file:foo", "file:///lv2.h"),
         ("lv2.h", "file:foo", "file:lv2.h"),
         ("s", "http://example.com", "http://example.com/s"),
+        (".", "file:", "file:"),
+        ("..", "file:", "file:"),
+        ("./", "file:", "file:"),
+        ("../", "file:", "file:"),
+        ("./.", "file:", "file:"),
+        ("../..", "file:", "file:"),
     ];
 
     for (relative, base, output) in examples {
         let base = Iri::parse(base).unwrap();
+        let output = Iri::parse(output).unwrap();
         match base.resolve(relative) {
-            Ok(result) => assert_eq!(
-                result.as_str(),
-                output,
-                "Resolving of {relative} against {base} is wrong. Found {result} and expecting {output}"
-            ),
+            Ok(result) => {
+                assert_eq!(
+                    result,
+                    output,
+                    "Resolving of {relative} against {base} is wrong. Found {result} and expecting {output}"
+                );
+                assert_eq!(result.scheme(), output.scheme());
+                assert_eq!(result.authority(), output.authority());
+                assert_eq!(result.path(), output.path());
+                assert_eq!(result.query(), output.query());
+                assert_eq!(result.fragment(), output.fragment());
+            }
             Err(error) => panic!(
                 "Resolving of {} against {} failed with error: {}",
                 relative, base, error
@@ -990,7 +1015,7 @@ fn test_relativize_iri() {
         (
             "http://example.com/foo/bar",
             "http://example.com/foo",
-            "/foo/bar",
+            "foo/bar",
         ),
         (
             "http://example.com/foo?bar",
@@ -1013,11 +1038,7 @@ fn test_relativize_iri() {
             "http://example.com/foo#baz",
             "#bar",
         ),
-        (
-            "http://example.com/foo/",
-            "http://example.com/foo/bar",
-            "/foo/",
-        ),
+        ("http://example.com/foo/", "http://example.com/foo/bar", "."),
         ("http://example.com/:", "http://example.com/foo", "/:"),
         ("http:", "http://example.com", "http:"),
         ("http:?foo", "http://example.com", "http:?foo"),
@@ -1046,14 +1067,17 @@ fn test_relativize_iri() {
         ("urn:ab", "urn:", "ab"),
         ("urn:isbn:foo", "urn:", "urn:isbn:foo"),
         ("urn:is/bn:foo", "urn:", "is/bn:foo"),
-        ("urn:.", "urn:", "."),
-        ("t:e/e/p", "t:e/s", "t:e/e/p"),
+        ("t:e/e/p", "t:e/s", "e/p"),
         ("htt:/foo/gp", "htt:/foo/", "gp"),
         ("htt:/gp", "htt:/", "gp"),
         ("x:", "x://foo", "x:"),
         ("x:", "x:02", "x:"),
         ("x:", "x:?foo", "x:"),
         ("x:", "x:?foo", "x:"),
+        ("http://example.com", "http://example.com#foo", ""),
+        ("http://example.com/a/", "http://example.com/a/b", "."),
+        ("http://example.com/a/?c", "http://example.com/a/b", ".?c"),
+        ("t:o//", "t:o/", "t:o//"),
     ];
 
     for (original, base, output) in examples {
@@ -1065,6 +1089,11 @@ fn test_relativize_iri() {
             actual, output,
             "Relativizing {original} against {base} gives {actual} and not {output}"
         );
+        assert_eq!(actual.scheme(), output.scheme());
+        assert_eq!(actual.authority(), output.authority());
+        assert_eq!(actual.path(), output.path());
+        assert_eq!(actual.query(), output.query());
+        assert_eq!(actual.fragment(), output.fragment());
         let resolved = base.resolve(actual.as_str()).unwrap();
         assert_eq!(
             resolved, original,
@@ -1076,18 +1105,22 @@ fn test_relativize_iri() {
 #[test]
 fn test_relativize_iri_fails() {
     let examples = [
-        "http://example.com/a/../b",
-        "http://example.com/a/..",
-        "http://example.com/./b",
-        "http://example.com/.",
+        ("http://example.com/a/../b", "http://example.com/s"),
+        ("http://example.com/a/..", "http://example.com/s"),
+        ("http://example.com/./b", "http://example.com/s"),
+        ("http://example.com/.", "http://example.com/s"),
+        ("urn:.", "urn:"),
     ];
 
-    let base = Iri::parse("http://example.com/s").unwrap();
-    for e in examples {
-        let e = Iri::parse(e).unwrap();
-        assert!(base.relativize(&e).is_err());
+    for (iri, base) in examples {
+        let iri = Iri::parse(iri).unwrap();
+        let base = Iri::parse(base).unwrap();
+        assert!(
+            base.relativize(&iri).is_err(),
+            "Relativize {iri} against {base} is not properly failing"
+        );
         // We make sure it's not possible to relativize
-        assert_ne!(base.resolve(e.as_str()).unwrap(), e);
+        assert_ne!(base.resolve(iri.as_str()).unwrap(), iri);
     }
 }
 
